#extension GL_OES_EGL_image_external : require
precision highp float;
uniform samplerExternalOES u_Texture;    // surface texture.
varying vec2 v_TexCoordinate;   // Interpolated texture coordinate per fragment.

uniform float planet_scale;
uniform float planet_factor;
uniform float shift_longitude;

// The entry point for our fragment shader.
void main()
{
    float PI = 3.14159265358979323846264;
    float HALF_PI = PI / 2.0;
    float TWO_PI = PI * 2.0;

    float cosphi0 = cos(planet_factor);
    float sinphi0 = sin(planet_factor);
    float r, g, b, y, u, v;
    vec4 zero = vec4(0, 0, 0, 0);
    vec4 one = vec4(1, 1, 1, 1);

    float xCoordinate = (v_TexCoordinate.x - 0.5) / planet_scale;
    float yCoordinate = (0.5 - v_TexCoordinate.y) / planet_scale;
    float rho = sqrt(xCoordinate * xCoordinate + yCoordinate * yCoordinate);
    float c = 2.0 * atan(rho);

    float sinc = sin(c);
    float cosc = cos(c);

    float lambda =  atan(xCoordinate * sinc, rho * cosc);
    float phi =  asin(yCoordinate * sinc / rho);
    float cosphi = cos(phi);

    float x1 = cos(lambda) * cosphi;
    float y1 = sin(lambda) * cosphi;
    float z1 = sin(phi);

    phi = asin(z1 * cosphi0 - x1 * sinphi0);
    lambda = mod(atan(y1, x1 * cosphi0 + z1 * sinphi0) + shift_longitude, TWO_PI);

    phi = (phi + HALF_PI) / PI;
    lambda = lambda / TWO_PI;

    vec2 coord = vec2(lambda, phi);

    gl_FragColor = texture2D(u_Texture, coord);
}